__package_name__ = 'alamari'
__version__ = '0.1.0'
__author__ = 'Siddharth Belbase'
__author_email__ = 'siddharthbelbase@gmail.com'
__description__ = (
    'A collection of utils that you might not require but might need.')
__url__ = 'https://github.com/sidbelbase/alamari'



from datetime import date


def to_integer(given_input: str):
    """
    Finds the number in a given input(should be a string with numbers) using regular expression.

    >>> to_integer('abc')
    0
    >>> to_integer('1')
    1
    >>> to_integer('abc123')
    123
    >>> to_integer('abc123abc')
    123
    >>> to_integer('abc123abc456')
    123,456

    Args:
        given_input (str): any text with numbers in it

    Returns:
        Number: returns number in a comma separated form if found else returns 0
    """
    from re import findall
    if isinstance(given_input, str):
        match = findall('\d+', given_input)
        if match:
            return ",".join(match)
        return 0  # default value
    return given_input  # if not string type


def to_roman(given_text: str) -> str:
    """Converts the given devanagiri text into roman form

    Args:
        given_text (str): devanagiri text

    Returns:
        str: romanized text
    """
    import nepali_roman as nr
    return nr.romanize_text(given_text)


def to_nepali_miti(english_date: date) -> date:
    """Converts english formmated date (international date) into nepali miti

    Args:
        english_date (date): international date

    Returns:
        date: returns nepali date in YY-M-D
    """
    from nepali_datetime import date as nepdate
    return nepdate.from_datetime_date(english_date)



from os import path

from setuptools import find_packages, setup

about = {}
with open('alamari/__about__.py') as fp:
    exec(fp.read(), about)


if path.isfile('README.md'):
    long_description = open('README.md').read()
else:
    long_description = 'See https://github.com/sidbelbase/alamari#readme'


if __name__ == '__main__':
    setup(
        name=about['__package_name__'],
        version=about['__version__'],
        author=about['__author__'],
        author_email=about['__author_email__'],
        description=about['__description__'],
        url=about['__url__'],
        license='MIT',
        packages=find_packages(exclude=[
            '*.__pycache__', 'venv', '.github']),
        long_description=long_description,
        long_description_content_type='text/markdown',
        install_requires=['arrow', 'nepali-datetime',
                          'nepali-roman', 'python-dateutil', 'requests'],
        setup_requires=[],
        keywords=['nepali-utils', 'made-in-nepal', 'nepal', 'nepali-developer',
                  'nepali-package', 'pluralize', 'devanagari', 'roman', 'nepal-time', 'helper-functions', 'utilities'],
        classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
        ],
        python_requires='>=3.6',
    )



from datetime import datetime
from typing import Optional, Union

import arrow

Number = Union[int, float, str]


def number(any_num: Number, padding: Optional[bool] = False) -> str:
    """Formats number in nepali counting form replacing the international format

    Args:
        any_num (Number): The number to be formatted
        padding (bool, optional): Defaults to False. If padding is required in any number less than 10.

    Syntax:
    >>> number(any_num, padding)

    Example:

    >>> number(any_num = 9, padding = True)
    09

    >>> number(any_num = 9.54, padding = True)
    09

    Returns:
        str: returns a string in comma separated number
    """
    int_part, *float_part = str(any_num).partition(".")
    separated_num = [int_part[x-2:x]
                     for x in range(-3, -len(int_part), -2)][::-1] + [int_part[-3:]]
    number_with_commas = ",".join(separated_num)
    if padding and int(int_part) < 10:
        return f"0{int_part}"
    return "".join([number_with_commas] + float_part)


def date(any_date: datetime) -> str:
    """Converts any given date into humanized form (UTC)

    Syntax:
    >>> date(any_date)

    Example:

    >>> date(any_date = 2021-06-02 05:55:55.185035)
    2 hours ago

    Args:
        any_date (datetime): any date with datetime format

    Returns:
        [type]: Returns date in humanized form
    """
    return arrow.get(any_date).to('US/Pacific').humanize()


def nepal_date(local_date: datetime) -> str:
    """Converts datetime into humanized form (NPT)

    Syntax:
    >>> nepal_date(local_date)

    Example:

    >>> nepal_date(local_date = 2021-06-02 05:55:55.185035)
    just now

    Args:
        local_date (datetime): any local date with datetime format

    Returns:
        [type]: Returns date in humanized form
    """
    return arrow.get(local_date).shift(minutes=-345).to('US/Pacific').humanize()



from datetime import datetime


def url_resolves(any_url: str) -> bool:
    """Checks if the given url resolves sucessfully

    Args:
        any_url (str): the full url of any media, article, content
    Returns:
        bool: returns False if it can't resolve the given link
    """
    from requests import get
    response = get(any_url)
    if response.status_code == 200:
        return True
    return False


def replace(given_text: str, sub_string: str, replacable_str: str) -> str:
    """Replace a substring with another string from a given text.

    Args:
        given_text (str): the full text where to be replaced
        sub_string (str): the string to be replaced
        replacable_str (str): the new replaced string

    Returns:
        str: stripped replaced text
    """
    from re import sub
    return sub(sub_string, replacable_str, given_text).strip()


def parse_date(given_string: str) -> datetime:
    """Parses datetimes from given string

    Args:
        given_string (str): string to parse from

    Returns:
        datetime: returns datetime object
    """
    from dateutil.parser import parse
    return parse(given_string)


def crop_image(image):
    # Image object : Pillow Image
    # crops the image from the center
    full_width, full_height = image.size
    width = min(image.size)
    height = min(image.size)
    return image.crop(((full_width - width) // 2, (full_height - height) // 2, (full_width + width) // 2, (full_height + height) // 2))


def ordinalize(given_number: int) -> str:
    """Ordinalize the number from the given number

    Args:
        given_number (int): integer number

    Example:
    >>> ordinalize(34)
    '34th'

    Returns:
        str: string in ordinal form
    """
    suffix = ["th", "st", "nd", "rd"]
    thenum = int(given_number)
    if thenum % 10 in [1, 2, 3] and thenum not in [11, 12, 13]:
        return f'{thenum}{suffix[thenum % 10]}'
    else:
        return f'{thenum}{suffix[0]}'


def pluralize(given_noun: str, quantity: int = 2, suffix: str = None) -> str:
    """Pluralize the given noun with suitable suffix

    Args:
        given_noun (str): string to be pluralized
        quantity (int): quantity to be pluralized (defaults to 2)
        suffix (str, optional): custom suffix to be used if not specified before (defaults to None)

    Returns:
        str: string in plural form
    """
    from re import search, sub
    if quantity > 1:
        if search('[sxz]$', given_noun):
            return sub('$', 'es', given_noun)
        elif search('[^aeioudgkprt]h$', given_noun):
            return sub('$', 'es', given_noun)
        elif search('[aeiou]y$', given_noun):
            return sub('y$', 'ies', given_noun)
        else:
            if suffix is not None:
                return given_noun + suffix
            return given_noun + 's'
    return given_noun
